package org.rcisoft.util;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.ExcelImportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import cn.afterturn.easypoi.excel.entity.ImportParams;
import cn.afterturn.easypoi.excel.entity.enmus.ExcelType;
import cn.afterturn.easypoi.excel.entity.params.ExcelExportEntity;
import cn.afterturn.easypoi.excel.export.ExcelExportService;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddressList;
import org.apache.poi.xssf.usermodel.XSSFDataValidation;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * @Author: gy
 * @Date: 2024/4/20 12:57
 * @Description: 动态列导出（非注解形式）
 */
@Slf4j
public class ExportToExcelUtil extends CyEpExcelUtil {

    public static <T> List<T> importExcelNotTitle(MultipartFile file, Integer titleRows, Integer headerRows, Class<T> pojoClass) {
        if (file == null) {
            return null;
        } else {
            ImportParams params = new ImportParams();
            List list = null;

            try {
                list = ExcelImportUtil.importExcel(file.getInputStream(), pojoClass, params);
                return list;
            } catch (Exception var7) {
                throw new CyServiceException(502, "模板格式错误");
            }
        }
    }


    /**
     * 为Excel表格添加下拉选项
     * 解决单元格在某个时间点只能访问有限数据量而导致无法显示下拉选项的问题
     * 适用于下拉选项总字节数大于 255 个字节的情况
     * 缺点：会创建一个隐藏的sheet用于存放下拉选项
     *
     * @param formulaString 下拉选项数据
     * @param workbook      Excel工作簿
     * @param sheet         设置下拉选项的工作表
     * @param firstRow      起始行
     * @param lastRow       终止行
     * @param firstCol      起始列
     * @param lastCol       终止列
     */
    public static void XSSFSetDropDownAndHidden(String[] formulaString, Workbook workbook, Sheet sheet, int firstRow, int lastRow, int firstCol, int lastCol) {
        // 计算下拉选项全部字节数
        int totalBytes = Arrays.stream(formulaString)
                .mapToInt(str -> str.getBytes(StandardCharsets.UTF_8).length)
                .sum();
        // 创建一个数据验证助手
        DataValidationHelper dvHelper = sheet.getDataValidationHelper();
        if (totalBytes > 255) {
            // 当总选项字节数大于 255 时创建隐藏工作表用于存放下拉选项
            // 创建sheet，写入枚举项
            //判断当前sheet不存在则创建
            Sheet hideSheet = workbook.getSheet("hiddenSheet");
            if (workbook.getSheet("hiddenSheet") == null) {
                hideSheet = workbook.createSheet("hiddenSheet");
            }

            for (int i = 0; i < formulaString.length; i++) {
                // 将下拉选项值存放在目标工作表下拉项目相同列
                hideSheet.createRow(i).createCell(firstCol).setCellValue(formulaString[i]);
            }
            // 创建名称，可被其他单元格引用
            //判断当前名称不存在则创建
            Name hiddenSheetName = workbook.getName("hidden");
            if (workbook.getName("hidden") == null) {
                hiddenSheetName = workbook.createName();
                hiddenSheetName.setNameName("hidden");
            }
            // 设置名称引用的公式
            // 使用像'A1：B1'这样的相对值会导致在Microsoft Excel中使用工作簿时名称所指向的单元格的意外移动，
            // 通常使用绝对引用，例如'$A$1:$B$1'可以避免这种情况。
            // 根据 firstCol 计算引用索引字母
            StringBuilder columnLetter = new StringBuilder();
            int columnIndex = firstCol;
            while (columnIndex >= 0) {
                int rem = columnIndex % 26;
                columnLetter.append((char) ('A' + rem));
                columnIndex = (columnIndex / 26) - 1;
            }
            columnLetter.reverse();
            hiddenSheetName.setRefersToFormula("hiddenSheet!$" + columnLetter + "$1:$" + columnLetter + "$" + formulaString.length);
            // 获取上文名称内数据
            DataValidationConstraint constraint = dvHelper.createFormulaListConstraint("hidden");
            // 设置下拉框位置
            CellRangeAddressList addressList = new CellRangeAddressList(firstRow, lastRow, firstCol, lastCol);
            DataValidation dataValidation = dvHelper.createValidation(constraint, addressList);
            // 处理Excel兼容性问题
            if (dataValidation instanceof XSSFDataValidation) {
                // 数据校验
                dataValidation.setSuppressDropDownArrow(true);
                dataValidation.setShowErrorBox(true);
            } else {
                dataValidation.setSuppressDropDownArrow(false);
            }
            // 作用在目标sheet上
            sheet.addValidationData(dataValidation);
            // 设置hiddenSheet隐藏
            int hideSheetIndex = workbook.getSheetIndex(hideSheet);
            workbook.setSheetHidden(hideSheetIndex, true);
        } else {
            // 当总选项字节数不大于 255 时使用传统方式创建下拉选项
            // 创建下拉列表约束
            DataValidationConstraint dvConstraint = dvHelper.createExplicitListConstraint(formulaString);
            // 设置数据验证的单元格区域（从firstRow行firstCol列到lastRow行lastCol列）
            CellRangeAddressList addressList = new CellRangeAddressList(firstRow, lastRow, firstCol, lastCol);
            // 创建数据验证对象并添加到工作表
            DataValidation validation = dvHelper.createValidation(dvConstraint, addressList);
            sheet.addValidationData(validation);
        }
    }


    // 导出动态列

    /**
     * entityList 表头
     * dataList 数据集合
     **/
    public static void exportExcel(HttpServletResponse response, List<ExcelExportEntity> entityList, List<Map<String, Object>> dataList) {
        // 1、创建一个Workbook（XSSFWorkbook）
        Workbook workbook = new XSSFWorkbook();
        // 3、定义标题和sheetName
        ExportParams exportParams = new ExportParams();
        exportParams.setStyle(ExcelExportStatisticStyler.class);
        // 默认未 ExcelType.HSSF导出失败
        exportParams.setType(ExcelType.XSSF);
        ExcelExportService service = new ExcelExportService();
        service.createSheetForMap(workbook, exportParams, entityList, dataList);
        downLoadExcel("导出统计", response, workbook);

    }

    public static void exportExcel(List<?> list, String title, String sheetName, Class<?> pojoClass, String fileName, HttpServletResponse response) {
        ExportParams exportParams = new ExportParams();
        exportParams.setStyle(ExcelExportStatisticStyler.class);
        String[] format = fileName.split("\\.");
        if (format[1].equals("xlsx")) {
            exportParams.setType(ExcelType.XSSF);
        }
        defaultExport(list, pojoClass, fileName, response, exportParams);
    }

    private static void defaultExport(List<?> list, Class<?> pojoClass, String fileName, HttpServletResponse response, ExportParams exportParams) {
        Workbook workbook = ExcelExportUtil.exportExcel(exportParams, pojoClass, list);
        if (workbook != null) {
            downLoadExcel(fileName, response, workbook);
        }

    }

    // 下载excel
    public static void downLoadExcel(String fileName, HttpServletResponse response, Workbook workbook) {
        ServletOutputStream outputStream = null;

        try {
            response.setCharacterEncoding("UTF-8");
            response.setHeader("content-Type", "application/vnd.ms-excel");
            response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
            outputStream = response.getOutputStream();
            workbook.write(outputStream);
        } catch (IOException var12) {
            throw new CyServiceException(var12.getMessage());
        } finally {
            try {
                if (outputStream != null) {
                    outputStream.close();
                }
            } catch (IOException var11) {
                log.info("context:" + var11);
            }
        }
    }
}
